import 'package:flutter/material.dart';
import 'package:flutter/foundation.dart';

class CustomAlertDialog<T> extends StatelessWidget {
  /// The (optional) title of the dialog is displayed in a large font at the top
  /// of the dialog.
  ///
  /// Typically a [Text] widget.
  final Widget title;

  /// Padding around the title.
  ///
  /// If there is no title, no padding will be provided. Otherwise, this padding
  /// is used.
  ///
  /// This property defaults to providing 24 pixels on the top, left, and right
  /// of the title. If the [content] is not null, then no bottom padding is
  /// provided (but see [contentPadding]). If it _is_ null, then an extra 20
  /// pixels of bottom padding is added to separate the [title] from the
  /// [actions].
  final EdgeInsetsGeometry? titlePadding;

  /// The (optional) content of the dialog is displayed in the center of the
  /// dialog in a lighter font.
  ///
  /// Typically, this is a [ListView] containing the contents of the dialog.
  /// Using a [ListView] ensures that the contents can scroll if they are too
  /// big to fit on the display.
  final Widget content;

  /// Padding around the content.
  ///
  /// If there is no content, no padding will be provided. Otherwise, padding of
  /// 20 pixels is provided above the content to separate the content from the
  /// title, and padding of 24 pixels is provided on the left, right, and bottom
  /// to separate the content from the other edges of the dialog.
  final EdgeInsetsGeometry? contentPadding;

  /// The (optional) set of actions that are displayed at the bottom of the
  /// dialog.
  ///
  ///
  /// These widgets will be wrapped in a [ButtonBar], which introduces 8 pixels
  /// of padding on each side.
  ///
  /// If the [title] is not null but the [content] _is_ null, then an extra 20
  /// pixels of padding is added above the [ButtonBar] to separate the [title]
  /// from the [actions].
  final List<Widget> actions;

  /// The semantic label of the dialog used by accessibility frameworks to
  /// announce screen transitions when the dialog is opened and closed.
  ///
  /// If this label is not provided, a semantic label will be infered from the
  /// [title] if it is not null.  If there is no title, the label will be taken
  /// from [MaterialLocalizations.alertDialogLabel].
  ///
  /// See also:
  ///
  ///  * [SemanticsConfiguration.isRouteName], for a description of how this
  ///    value is used.
  final String semanticLabel;

  /// The (optional) horizontal separator used between title, content and
  /// actions.
  ///
  /// If this divider is not provided a [Divider] is used with [height]
  /// property is set to 0.0
  final Widget? divider;

  ///[divider] is not displayed if set to false. Default is set to true.
  final bool isDividerEnabled;

  /// Creates an alert dialog.
  ///
  /// Typically used in conjunction with [showDialog].
  ///
  /// The [contentPadding] must not be null. The [titlePadding] defaults to
  /// null, which implies a default that depends on the values of the other
  /// properties. See the documentation of [titlePadding] for details.
  const CustomAlertDialog({
    Key? key,
    required this.title,
    this.titlePadding,
    required this.content,
    this.contentPadding = const EdgeInsets.fromLTRB(24.0, 20.0, 24.0, 24.0),
    this.actions = const [],
    this.semanticLabel = '',
    this.divider = const Divider(
      height: 0.0,
    ),
    this.isDividerEnabled = true,
  }) : super(key: key);
  @override
  Widget build(BuildContext context) {
    return Dialog(
      child: Semantics(
        namesRoute: true,
        label: semanticLabel,
        child: Column(
          mainAxisSize: MainAxisSize.min,
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: <Widget>[
            Padding(
              padding: titlePadding ??
                  EdgeInsets.fromLTRB(
                    24.0,
                    24.0,
                    24.0,
                    isDividerEnabled ? 20.0 : 0.0,
                  ),
              child: DefaultTextStyle(
                style: Theme.of(context).textTheme.headline6!,
                child: Semantics(
                  namesRoute: true,
                  child: title,
                ),
              ),
            ),
            if (isDividerEnabled && divider != null) divider!,
            Flexible(
              child: Padding(
                padding: contentPadding ?? EdgeInsets.all(4),
                child: DefaultTextStyle(
                  style: Theme.of(context).textTheme.subtitle1!,
                  child: content,
                ),
              ),
            ),
            if (isDividerEnabled && divider != null) divider!,
            if (actions.isNotEmpty)
              ButtonBar(
                children: actions,
              )
          ],
        ),
      ),
    );
  }
}
